; #UDFFILE# ====================================================================================================================
; Name...........: ZLib UDFs
; Description ...: UDFs to use some zlib functions with AutoIt
; Author ........: monoceres ( http://www.autoitscript.com/forum/index.php?showuser=23930 )
; Modified.......: Prog@ndy ( http://www.autoitscript.com/forum/index.php?showuser=31965 )
; Remarks .......: The functions do not have extra notes. This header is valid for all functions.
; ===============================================================================================================================
Global Const $Z_OK = 0
Global Const $Z_STREAM_END = 1
Global Const $Z_NEED_DICT = 2
Global Const $Z_ERRNO = (-1)
Global Const $Z_STREAM_ERROR = (-2)
Global Const $Z_DATA_ERROR = (-3)
Global Const $Z_MEM_ERROR = (-4)
Global Const $Z_BUF_ERROR = (-5)
Global Const $Z_VERSION_ERROR = (-6)

Global Const $Z_NO_COMPRESSION = 0
Global Const $Z_BEST_SPEED = 1
Global Const $Z_BEST_COMPRESSION = 9
Global Const $Z_DEFAULT_COMPRESSION = (-1)

Global $Zlib_Dll = 0

#cs
;~ #include "zlib udf.au3"
_Zlib_Startup()
$bin=StringToBinary("hahahahahahahahahahahehe")
$binlen=BinaryLen($bin)

$compressed=_Zlib_CompressBinary($bin)

$uncompressed=_Zlib_UncompressBinary($compressed,$binlen)

MsgBox(64,BinaryToString($bin),"Original:"&@CRLF&$bin&@CRLF&@CRLF&"Compressed: "&@CRLF&$compressed&@CRLF&@CRLF&"Uncompressed: "&@CRLF&$uncompressed)
_Zlib_Shutdown()
#ce

; Returns the Zlib version as a string, i.e 3.2.1
Func _Zlib_Version()
	Local $call = DllCall($Zlib_Dll, "str:cdecl", "zlibVersion")
	If @error Then Return SetError(1,0,"")
	Return $call[0]
EndFunc   ;==>_Zlib_Version

; Closes the zlib dll
Func _Zlib_Shutdown()
	DllClose($Zlib_Dll)
	$Zlib_Dll = -1
EndFunc   ;==>_Zlib_Shutdown

; Opens the zlib dll, default name is zlibwapi.dll
Func _Zlib_Startup($Filename = "zlib1.dll")
	If $Zlib_Dll <> -1 And _Zlib_Version()<>"" Then Return True
	$Zlib_Dll = DllOpen($Filename)
	Return $Zlib_Dll <> -1
EndFunc   ;==>_Zlib_Startup

; An implementation of the Adler32 checksum included in the zlib lib
Func _Zlib_CalculateAdler32($DataPtr, $DataSize)
	Local Const $ADLER32_BASE = 65521
	$call = DllCall($Zlib_Dll, "ulong:cdecl", "adler32", "ulong", $ADLER32_BASE, "ptr", $DataPtr, "int", $DataSize)
	If @error Then Return SetError(1,0,0)
	Return $call[0]
EndFunc   ;==>_Zlib_CalculateAdler32


; Decompresses data, you need to know how large the decompressed data will be.
Func _Zlib_Uncompress($CompressedPtr, $CompressedSize, $UncompressedPtr, $UncompressedSize)
	$call = DllCall($Zlib_Dll, "uint:cdecl", "uncompress", "ptr", $UncompressedPtr, "ulong*", $UncompressedSize, "ptr", $CompressedPtr, "ulong", $CompressedSize)
	If @error Then Return SetError(1,0,0)
	Return $call[0]
EndFunc   ;==>_Zlib_Uncompress

; Compresses data, the output buffer have to be at least InputBuffer + 0.1 % + 12 byte
Func _Zlib_Compress($InBufferPtr, $InBufferSize, $OutBufferPtr, ByRef $OutBufferSize, $CompressionLevel = $Z_DEFAULT_COMPRESSION)
	Local $call = DllCall($Zlib_Dll, "uint:cdecl", "compress2", "ptr", $OutBufferPtr, "ulong*", $OutBufferSize, "ptr", $InBufferPtr, "ulong", $InBufferSize, "int", $CompressionLevel)
	If @error Then Return SetError(1,0,0)
	$OutBufferSize = $call[2]
	Return $call[0]
EndFunc   ;==>_Zlib_Compress

; Compresses binary data
Func _Zlib_CompressBinary(ByRef Const $binary, $CompressionLevel = $Z_DEFAULT_COMPRESSION)
	Local $InByteArr = DllStructCreate("byte[" & BinaryLen($binary) & "]")
	DllStructSetData($InByteArr, 1, $binary)

	Local $OutByteArr = DllStructCreate("byte[" & _ZLib_CompressBound(BinaryLen($binary))+1 & "]")
	Local $OutByteArrSize = DllStructGetSize($OutByteArr)

	Local $ret = _Zlib_Compress(DllStructGetPtr($InByteArr), DllStructGetSize($InByteArr), DllStructGetPtr($OutByteArr), $OutByteArrSize, $CompressionLevel)

	If $ret <> 0 Then Return SetError($ret,0,"")

	Local $CompressedBuffer = DllStructCreate("byte[" & $OutByteArrSize & "]", DllStructGetPtr($OutByteArr))

	SetExtended($OutByteArrSize)
	Return DllStructGetData($CompressedBuffer, 1)
EndFunc   ;==>_Zlib_CompressBinary

Func _ZLib_CompressBound($iSize)
	; Author: Prog@ndy
	Local $call = DllCall($Zlib_Dll, "uint:cdecl", "compressBound", "ulong", $iSize)
	If @error Then Return SetError(1,0,0)
	Return $call[0]
EndFunc

; Decompresses binary data, you need to know the binary length of the decompressed data
Func _Zlib_UncompressBinary(ByRef Const $compressedbinary, $UncompressedBinaryLength)
	Local $InByteArr = DllStructCreate("byte[" & BinaryLen($compressedbinary) + 1 & "]")
	DllStructSetData($InByteArr, 1, $compressedbinary)

	Local $OutByteArr = DllStructCreate("byte[" & $UncompressedBinaryLength & "]")

	Local $ret = _Zlib_Uncompress(DllStructGetPtr($InByteArr), DllStructGetSize($InByteArr), DllStructGetPtr($OutByteArr), DllStructGetSize($OutByteArr))

	If $ret <> 0 Then Return SetError($ret,0,"")

	Return DllStructGetData($OutByteArr, 1)
EndFunc   ;==>_Zlib_UncompressBinary
